///////////////////////////////////////////////////////////////////////////////
//
// TRIPjxp Programming Examples
//
// Copyright (C) 2009 Tieto Sweden AB - All Rights Reserved
//
///////////////////////////////////////////////////////////////////////////////
//
// Modification History:
//		TF	Jun 5, 2009	Created
//              TF      Jul 1, 2009     Show use of TRIP date formatting.
//
///////////////////////////////////////////////////////////////////////////////

package com.tietoenator.trip.jxp.examples.data;

import com.tietoenator.trip.jxp.TdbException;
import com.tietoenator.trip.jxp.TdbFieldType;
import com.tietoenator.trip.jxp.TdbMessageProvider;
import com.tietoenator.trip.jxp.data.TdbComponent;
import com.tietoenator.trip.jxp.data.TdbRecord;
import com.tietoenator.trip.jxp.data.TdbTuple;
import com.tietoenator.trip.jxp.data.TdbTupleList;
import com.tietoenator.trip.jxp.data.TdbDateField;
import com.tietoenator.trip.jxp.database.TdbDatabaseDesign;
import com.tietoenator.trip.jxp.database.TdbFieldDesign;
import com.tietoenator.trip.jxp.session.TdbSession;

import com.tietoenator.trip.jxp.examples.util.Console;
import com.tietoenator.trip.jxp.examples.util.SessionFactory;

import java.util.Date;
import java.text.SimpleDateFormat;
import java.util.Locale;



/**
 * Demonstrates the use of the classes TdbTupleList and TdbTuple for reading
 * and manipulating tupled fields in a TdbRecord.
 * @since TRIPjxp 1.3-0
 */
public class Tuples
{
   private TdbSession m_session;
   private TdbDatabaseDesign m_db = null;
   private String m_strRecordName = null;

   Tuples() throws TdbException
   {
      // Create a session on a physical server (no grid support)
      m_session = SessionFactory.newInstance("**** Tuple list example ****", false);
   }

   // Show any success message that might be forthcoming from the server

   void showMessages(TdbMessageProvider prv)
   {
      for (String msg : prv.messageList())
      {
         System.out.println(msg);
      }
      prv.resetMessages();
   }

   boolean verifyField(TdbDatabaseDesign db, String fieldName, TdbFieldType type)
   {
      TdbFieldDesign fd;
      if ((fd = db.getFieldByName(fieldName)) == null)
      {
         System.out.println("Required field \""+fieldName+"\" does not exist.");
         return false;
      }

      if (fd.getTypeId() != type)
      {
         System.out.println("Required field \""+fieldName+"\" was of an invalid type.");
         return false;
      }

      return true;
   }

   String formatTime(java.util.Date d)
   {
      SimpleDateFormat tf = new SimpleDateFormat("HH:mm:ss");
      return tf.format(d);
   }

   /**
    * Returns a Date in the current TRIP date format.
    */
   String formatDate(java.util.Date d) throws TdbException
   {
      return m_session.getCurrentDateform().formatDate(d);
   }

   /**
    * Verifies that the specified database exists and creates it if it doesn't.
    */
   boolean verifyDatabase(String dbname) throws TdbException
   {
      boolean bExists = true;
      TdbDatabaseDesign db = null;

      try
      {
         db = new TdbDatabaseDesign(m_session);
         db.get(dbname);
      }
      catch (TdbException x)
      {
         if (x.getCode() == 610)
            bExists = false;
         else
         {
            System.out.println(x.getDebugMessage());
            return false;
         }
      }

      if (bExists)
      {
         if (!verifyField(db,"R_INV_LOCATION",TdbFieldType.PhraseField)) return false;
         if (!verifyField(db,"T_PROD_NAME",TdbFieldType.PhraseField)) return false;
         if (!verifyField(db,"T_PROD_PRICE",TdbFieldType.NumberField)) return false;
         if (!verifyField(db,"T_INV_COUNT",TdbFieldType.IntegerField)) return false;
         if (!verifyField(db,"T_INV_DATE",TdbFieldType.DateField)) return false;
         if (!verifyField(db,"T_INV_TIME",TdbFieldType.TimeField)) return false;
      }
      else
      {

         Console stdin = new Console();
         String strYN = null;
         do
         {
            strYN = stdin.readln("Database does not exist. Create it?","Y");
            if (strYN != null)
            {
               if (strYN.equalsIgnoreCase("N"))
                  return false;
               else if (strYN.equalsIgnoreCase("Y") == false)
                  strYN = null;
            }
         }
         while (strYN == null);


         String location = null;
         do
         {
            location = stdin.readln("File location (logical name)", "");
         }
         while (location == null);

         // Create database design. For more info about database creation,
         // please refer to the "Create" example program.

         db = new TdbDatabaseDesign(m_session);

         db.setBafFile(location + ":" + dbname + ".BAF");
         db.setBifFile(location + ":" + dbname + ".BIF");
         db.setVifFile(location + ":" + dbname + ".VIF");

         TdbFieldDesign f = new TdbFieldDesign();
         f.setName("R_INV_LOCATION");
         f.setType("PHRASE");
         f.setRecordNameField(true);
         db.addField(f);

         f = new TdbFieldDesign();
         f.setName("T_PROD_NAME");
         f.setType("PHRASE");
         db.addField(f);

         f = new TdbFieldDesign();
         f.setName("T_PROD_PRICE");
         f.setType("NUMBER");
         db.addField(f);

         f = new TdbFieldDesign();
         f.setName("T_INV_COUNT");
         f.setType("INTEGER");
         db.addField(f);

         f = new TdbFieldDesign();
         f.setName("T_INV_DATE");
         f.setType("DATE");
         db.addField(f);

         f = new TdbFieldDesign();
         f.setName("T_INV_TIME");
         f.setType("TIME");
         db.addField(f);
 
         db.put(dbname);
         showMessages(db);
      }

      m_db = db;
      return true;
   }

   /**
    * Create a new record with tuples
    */
   void createTuples(Console stdin) throws TdbException
   {
      Date now = new Date();
      TdbRecord rec = new TdbRecord(m_session,m_db,false);

      // Create fields in the record object. NOTE: The boolean constructor
      // argument 'create' does not do this as that only creates field
      // templates for retrieval.
      TdbComponent head = rec.getHead();
      head.createField(m_db.getFieldByName("R_INV_LOCATION"));
      head.createField(m_db.getFieldByName("T_PROD_NAME"));
      head.createField(m_db.getFieldByName("T_PROD_PRICE"));
      head.createField(m_db.getFieldByName("T_INV_COUNT"));
      head.createField(m_db.getFieldByName("T_INV_DATE"));
      head.createField(m_db.getFieldByName("T_INV_TIME"));

      // Read inventory location - also used as record name.
      // Loop until we have a record name not in use.
      m_strRecordName = null;
      while (m_strRecordName == null)
      {
         m_strRecordName = stdin.readln("Inventory Location", "WarehouseOne");

         try
         {
            TdbRecord rchk = new TdbRecord(m_session,m_db,true);
            rchk.setRecordName(m_strRecordName);
            rchk.get();
            rchk = null;

            System.err.println("Record alreay exists. Please specify another location name.");
            m_strRecordName = null;
         }
         catch (TdbException x)
         {
            // Assume that the error indicates a non-existing record.
            // A real application should check x.getCode() for proper
            // error control.
         }
      }

       // Set the record name
      rec.setRecordName(m_strRecordName);

      // Now create the tuple list
      TdbTupleList tlist = new TdbTupleList(m_session, head, "T_PROD_NAME;T_PROD_PRICE;T_INV_COUNT;T_INV_DATE;T_INV_TIME");

      // Add some data for the various supported tuple types.
      TdbTuple t = tlist.append();
      t.setValue(0, "pencil, ball-point");
      t.setValue(1, Double.toString(2.50));
      t.setValue(2, Integer.toString(17));
      t.setValue(3, formatDate(now));
      t.setValue(4, formatTime(now));

      t = tlist.append();
      t.setValue(0, "eraser");
      t.setValue(1, Double.toString(1.15));
      t.setValue(2, Integer.toString(9));
      t.setValue(3, formatDate(now));
      t.setValue(4, formatTime(now));

      t = tlist.append();
      t.setValue(0, "binder, A4, black");
      t.setValue(1, Double.toString(3.45));
      t.setValue(2, Integer.toString(21));
      t.setValue(3, formatDate(now));
      t.setValue(4, formatTime(now));

      rec.commit();
      m_db.index(false,true);
   }

   void showTuples() throws TdbException
   {
      TdbRecord rec = getRecord();
      TdbTupleList tlist = createTupleList(rec);

      System.out.println("RECORD: \""+rec.getRecordName()+"\"");
      for (int i=0;i<tlist.getRowCount();i++)
      {
         TdbTuple t = tlist.get(i);
         System.out.println("----[Tuple #"+Integer.toString(i+1)+"]------------------");

         for (int j=0;j<tlist.getColumnCount();j++)
         {
            System.out.println("  "+tlist.getColumnName(j)+": "+t.getValue(j));
         }
      }

      System.out.println();
   }

   void removeTuples() throws TdbException
   {
      System.out.println("Removing middle tuple.");

      TdbRecord rec = getRecord();
      TdbTupleList tlist = createTupleList(rec);

      // Note: tuple #2 has index 1
      tlist.remove(1);

      // Save changes
      rec.commit();
      rec = null;
   }

   void showTupleCount() throws TdbException
   {
      TdbRecord rec = getRecord();
      TdbTupleList tlist = createTupleList(rec);

      System.out.println("Record "+m_strRecordName+" now has "+tlist.getRowCount()+" tuples.");
   }

   /**
    * Returns the record for the previously specified record name.
    * @return TdbRecord object
    * @throws com.tietoenator.trip.jxp.TdbException
    */
   private TdbRecord getRecord()  throws TdbException
   {
      TdbRecord rec = new TdbRecord(m_session,m_db,true);
      rec.setRecordName(m_strRecordName);
      rec.get();
      return rec;
   }

   /**
    * Creates a tuple list object for the tuple structure in the test database.
    * @param rec TdbRecord object to create the tuple list for.
    * @return TdbTupleList object
    * @throws com.tietoenator.trip.jxp.TdbException
    */
   private TdbTupleList createTupleList(TdbRecord rec) throws TdbException
   {
      return new TdbTupleList(m_session, rec.getHead(), "T_PROD_NAME;T_PROD_PRICE;T_INV_COUNT;T_INV_DATE;T_INV_TIME");
   }

   /**
    * Main entry point
    * @param args Not used
    */
   public static void main(String[] args)
   {
      try
      {
         Tuples prg = new Tuples();

         Console stdin = new Console();
         String db = null;

         // Get a valid database name, and set the list of
         // retrieval fields from it
         while (db == null)
         {
            db = stdin.readln("Database", "JXPTUPLES");
         }

         if (!prg.verifyDatabase(db))
         {
            System.out.println("Database verification failed - program aborted!");
            return;
         }

         prg.createTuples(stdin);
         prg.showTuples();
         prg.showTupleCount();
         prg.removeTuples();
         prg.showTupleCount();
      }
      catch (TdbException e)
      {
         System.out.println(e.getDebugMessage());
         e.printStackTrace();
      }
      catch (Exception e)
      {
         e.printStackTrace();
      }
   }
}
