///////////////////////////////////////////////////////////////////////////////
//
// TRIPjxp Programming Examples
//
// Copyright (C) 2018 infinIT Services GmbH
//
///////////////////////////////////////////////////////////////////////////////
//
// Modification History:
//        TF    Jul 19, 2018    Created
//        TF    Apr 24, 2019    Adapted to TRIP 8.0
//
///////////////////////////////////////////////////////////////////////////////
package com.tietoenator.trip.jxp.examples.facet;

import com.tietoenator.trip.jxp.TdbException;
import com.tietoenator.trip.jxp.TdbMessageProvider;
import com.tietoenator.trip.jxp.examples.util.Console;
import com.tietoenator.trip.jxp.examples.util.SessionFactory;
import com.tietoenator.trip.jxp.ccl.TdbCclCommand;
import com.tietoenator.trip.jxp.data.TdbSearch;
import com.tietoenator.trip.jxp.data.TdbSearchSet;
import com.tietoenator.trip.jxp.facet.TdbFacet;
import com.tietoenator.trip.jxp.facet.TdbFacetSet;
import com.tietoenator.trip.jxp.facet.TdbFacetSortOrder;
import com.tietoenator.trip.jxp.facet.TdbFacetValue;
import com.tietoenator.trip.jxp.facet.TdbFieldTermFacet;
import com.tietoenator.trip.jxp.session.TdbSession;

/**
 * Example of how to use baselining with the facet classes in TRIPjxp.
 *
 * <p>Requires TRIPsystem 7.2 or later in order to work. Recommended
 * TRIPsystem version is 8.0 or later.</p>
 */
public class FacetBaseline
{
    private TdbSession m_session;
    private TdbSearch m_search;
    
    public FacetBaseline()
        throws TdbException
    {
        // Create a session on a physical server (no grid support)
        m_session = SessionFactory.newInstance("**** TRIP Facet Baseline Example ****", false);
        m_search = new TdbSearch(m_session);
    }
    
    public void close()
    {
        if (m_session != null)
        {
            
            try
            {
                m_session.logout();
            }
            catch (TdbException ignore)
            {
            }
            finally
            {
                m_session = null;
            }
        }
    }
    
    
    // Show any success message that might be forthcoming from the server
    
    private static void showMessages(TdbMessageProvider prv)
    {
        for (String msg : prv.messageList())
        {
            System.out.println(msg);
        }
        prv.resetMessages();
    }
    
    
    private void printFacetValues(TdbFacet f, int from, int to)
        throws TdbException
    {
        int first, last;
		
        // The range of values to show in the user interface is handled
        // here, not when requesting the facet via TdbFacetSet.get().
		
		if (from > f.getValueCount()) {
            return;
        }
        
        first = from > 0 ? from : 1;
		if (to > 0 && to <= f.getValueCount() && to >= first) {
			last = to;
		} else {
            last = f.getValueCount();
        }
		
        System.out.println();
        System.out.print(f.getName().toUpperCase());
		System.out.print(String.format(" (showing %d/%d values",
                           last-first+1,f.getValueCount()));
        if (f.getUpdateValueCount() < f.getValueCount())
        {
            System.out.println(String.format(", %d/%d non-zero counts):",
                                             f.getUpdateValueCount(),
                                             f.getValueCount()));
        }
        else
        {
            System.out.println("):");
        }
        
        for (int index=from-1;
             index < to;
             index++)
        {
            TdbFacetValue v = f.getValue(index);
            
            System.out.print(String.format("T=%02d  <%02d>  ",
                                           index+1,
                                           v.getLongRecordCount()));
            
            // The isBaselineValid property can be used to check if it makes sense
            // to retrieve and display the baseline count.
            if (v.isBaselineValid())
            {
                System.out.print(String.format("<%02d>  ",
                                               v.getLongBaselineCount()));
            }
            
            System.out.println(v.toString());
        }
    }
    

    /**
     * Add a TdbFacet that will be used to register a new baseline.
     */
    private void createFacet(TdbFacetSet facetset, String name, int setno)
        throws TdbException
    {
        TdbFieldTermFacet facet1 = new TdbFieldTermFacet(name, "#");
        facet1.setRegisterBaseline(true);
        facet1.setBaselineKey("ALICE-"+name);
        
        facet1.setSearchSet(setno);
        facet1.setSortOrder(TdbFacetSortOrder.ValueDescending);
        facet1.setBaselineAtEnd(true);
        facetset.add(facet1);
    }
    
    /**
     * Add a TdbFacet that will be used to refine an existing baseline.
     */
    private void refineFacet(TdbFacetSet facetset, String name, int setno)
        throws TdbException
    {
        TdbFieldTermFacet facet1 = new TdbFieldTermFacet(name, "#");
        facet1.setUseBaseline(true);
        facet1.setBaselineKey("ALICE-"+name);
    
        facet1.setSearchSet(setno);
        facet1.setSortOrder(TdbFacetSortOrder.ValueDescending);
        facet1.setBaselineAtEnd(true);
        facetset.add(facet1);
    }

    /**
     * Add a TdbFacet that will be used to access existing, possibly
     * refined, baseline without refining it further at this point.
     */
    private void openFacet(TdbFacetSet facetset, String name)
        throws TdbException
    {
        TdbFieldTermFacet facet1 = new TdbFieldTermFacet(name);
        facet1.setOpenExisting(true);           // Open only
        facet1.setUseBaseline(true);            // Required for open
        facet1.setBaselineKey("ALICE-"+name);   // Needed if previously set
        facetset.add(facet1);
        
        // NB: The call to setUseBaseline(true) can be left out here since
        // this is applied as part of the call to setOpenExisting(true). It
        // was made explict here to illustrate the relationship to baselines.
    }

    /**
     * Add a TdbFacet that will be used to access existing, possibly
     * refined, baseline without refining it further at this point.
     */
    private void resetFacet(TdbFacetSet facetset, String name)
        throws TdbException
    {
        TdbFieldTermFacet facet1 = new TdbFieldTermFacet(name);
        facet1.setResetBaseline(true);          // Indicate a reset
        facet1.setBaselineKey("ALICE-"+name);   // Needed if previously set
        facetset.add(facet1);
		
		// The result of a reset operation is the same as for openExisting,
		// except that the baseline counts have been set to the counts that
		// the last refinement resulted in.
		
		// NB: For a baseline reset, the sort parameters (via the
		// setSortOrder and setBaselineAtEnd methods) are disregarded.
		// The previously specified sort order will be retained.
    }
    
    /**
     * Shows how to register new facet with baseline support.
     */
    private void processFirst() throws TdbException
    {
        // A common scenario is to first open a database and perhaps
        // do a (full text) search.
		
        m_search.execute("BASE ALICE");
        m_search.execute("FIND ALICE");
		int setno = m_search.getLastSearchSet().getSearchId();
        
        // Retrieve the facet values and set them as baseline
        TdbFacetSet facetset = new TdbFacetSet(m_session);
        createFacet(facetset,"SPEAKER",setno);
        createFacet(facetset,"PERSON",setno);
        facetset.get();
        
        // Show first round of results
        for (int facetIdx=0;
             facetIdx<facetset.getFacetCount();
             facetIdx++)
        {
            printFacetValues(facetset.getFacet(facetIdx),1,10);
        }
        
        // Remember to close the facet set immediately when you're done with
        // it. Note that the close operation does not remove baselines.
        facetset.close();
    }
    
    
    /**
     * Shows how to refine facet with baseline support.
     */
    private void processRefine() throws TdbException
    {
        // Assume the user has selected a facet in PERSON, so
        // here we emulate a search refinement scenario.
        
        TdbSearchSet sset = m_search.getLastSearchSet();
        m_search.execute("FIND S="+Integer.toString(sset.getSearchId())+
                         " AND PERSON='WHITE KING'");
		int setno = m_search.getLastSearchSet().getSearchId();
        
        // Issue a new identical facet request and tell it to use the previous
        // one as a baseline. Any facet values missing from the new set will
        // be inserted from the baseline.
        
        TdbFacetSet facetset = new TdbFacetSet(m_session);
        refineFacet(facetset,"SPEAKER",setno);
        refineFacet(facetset,"PERSON",setno);
        facetset.get();

        // NB: Instead of calling refineFacet on the PERSON facet, we could
        // have used openFacet instead. That would leave all values and counts
        // exactly as they were. But because we would like to see the counts
        // reflecting the last search, we call refine on all facets.
        
        System.out.println();
        System.out.println("Refined facets with baseline information (range 1-10)");
        
        // Show second round of results
        for (int facetIdx=0;
             facetIdx<facetset.getFacetCount();
             facetIdx++)
        {
            printFacetValues(facetset.getFacet(facetIdx),1,10);
        }
        
        // Remember to close the facet set immediately when you're done with
        // it. Note that the close operation does not remove baselines.
        facetset.close();
    }
    

    /**
     * This method illustrates how to use a previousy registered baseline
     * facet without having to re-issue the operation that generated it.
     *
     * The scenario for this is typically when requesting additional ranges
     * of values, e.g. in response to the user scrolling down the list of
     * values.
     */
    private void processExisting() throws TdbException
    {
        TdbFacetSet facetset = new TdbFacetSet(m_session);
        openFacet(facetset,"SPEAKER");
        openFacet(facetset,"PERSON");
        facetset.get();
        
        System.out.println();
        System.out.println("Existing facets with baseline information (range 11-20)");
        
        // Show results from last refinement, selecting range 11-20.
        for (int facetIdx=0;
             facetIdx<facetset.getFacetCount();
             facetIdx++)
        {
            printFacetValues(facetset.getFacet(facetIdx),11,20);
        }
        
        // Remember to close the facet set immediately when you're done with
        // it. Note that the close operation does not remove baselines.
        facetset.close();
    }
    
    /**
     * This method illustrates how to use a previousy registered baseline
     * facet without having to re-issue the operation that generated it.
     *
     * The scenario for this is typically when requesting additional ranges
     * of values, e.g. in response to the user scrolling down the list of
     * values.
     */
    private void processResetBaseline() throws TdbException
    {
        TdbFacetSet facetset = new TdbFacetSet(m_session);
        resetFacet(facetset,"SPEAKER");
        resetFacet(facetset,"PERSON");
        facetset.get();
        
        System.out.println();
        System.out.println("Facets with reset baseline information (range 1-10)");
        
        // Show results from last refinement, selecting range 11-20.
        for (int facetIdx=0;
             facetIdx<facetset.getFacetCount();
             facetIdx++)
        {
            printFacetValues(facetset.getFacet(facetIdx),1,10);
        }
        
        // Remember to close the facet set immediately when you're done with
        // it. Note that the close operation does not remove baselines.
        facetset.close();
    }
    
	
    
    /**
     * Illustrates how to delete all baselines for specified facets if the
     * TdbFacetSet that was used to create or last access them is no longer
     * available.
     */
    private void closeAllBaselines() throws TdbException
    {
        TdbFacetSet facetset = new TdbFacetSet(m_session);
        openFacet(facetset,"SPEAKER");
        openFacet(facetset,"PERSON");
        facetset.get();
        facetset.removeBaselines();
        facetset.close();
    }

    
    public static void main(String[] args)
    {
        FacetBaseline app = null;
        Console stdin = new Console();
        try
        {
            app = new FacetBaseline();
            app.processFirst();
            app.processRefine();
            app.processExisting();
            app.processResetBaseline();
            app.closeAllBaselines();
        }
        catch (TdbException tx)
        {
            System.err.println(tx.getDebugMessage());
        }
        catch (Exception ox)
        {
            ox.printStackTrace(System.err);
        }
        finally
        {
            if (app != null)
                app.close();
        }
    }
}

